/*
 * iopending - Print a plot for the number of pending disk I/O events.
 *             Written using DTrace (Solaris 10 3/05).
 *
 * This is measuring disk events that have made it past system caches.
 * By plotting a distribution graph of the number of pending events, the
 * "serialness" or "parallelness" of disk behaviour can be distinguished.
 *
 * $Id: iopending 3 2007-08-01 10:50:08Z brendan $
 *
 * FIELDS:
 *		value		number of pending events, 0 == idle
 *		count		number of samples @ 1000 Hz
 *		load		1 min load average
 *		disk_r		total disk read Kbytes for sample
 *		disk_w		total disk write Kbytes for sample
 * 
 * SEE ALSO: iosnoop, iotop
 *
 * IDEA: Dr Rex di Bona (Sydney, Australia)
 *
 * COPYRIGHT: Copyright (c) 2005, 2006 Brendan Gregg.
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 *
 * Author: Brendan Gregg  [Sydney, Australia]
 *
 * 01-Nov-2005	Brendan Gregg	Created this.
 * 20-Apr-2006	   "      "	Last update.
 * 13-Nov-2008	Tom Erickson	(adapted for Chime)
 */

/*
 * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
 */

inline int FILTER = !(??ALL??);
inline string DEVICE = "??DEVICE??";
inline string FILENAME = "??FILENAME??";
inline string MOUNT = "??MOUNT??";
inline int OPT_device = (DEVICE != "");
inline int OPT_file = (FILENAME != "");
inline int OPT_mount = (MOUNT != "");
inline int MAX_PENDING = 32;	/* max pending value */
 
/*
 * Print header
 */
dtrace:::BEGIN 
{
	/* starting values */
	disk_r = 0;
	disk_w = 0;
	pending = 0;
}

/*
 * Check event is being traced
 */
io:genunix::start,
io:genunix::done 
{
	/* default is to trace unless filtering, */
	this->ok = FILTER ? 0 : 1;

	/* check each filter, */
	(OPT_device == 1 && DEVICE == args[1]->dev_statname)? this->ok = 1 : 1;
	(OPT_file == 1 && FILENAME == args[2]->fi_pathname) ? this->ok = 1 : 1;
	(OPT_mount == 1 && MOUNT == args[2]->fi_mount)  ? this->ok = 1 : 1;
}

/*
 * Store entry details
 */
io:genunix::start
/this->ok/
{
	/* track bytes */
	this->reads = args[0]->b_flags & B_READ ? args[0]->b_bcount : 0;
	this->writes = args[0]->b_flags & B_READ ? 0 : args[0]->b_bcount;
	@disk_r = sum(this->reads);
	@disk_w = sum(this->writes);

	/* increase event pending count */
	pending++;
}

/*
 * Process and Print completion
 */
io:genunix::done
/this->ok/
{
	/* decrease event pending count */
	pending--;
}

/*
 * Prevent pending from underflowing
 * this can happen if this program is started during disk events.
 */
io:genunix::done
/pending < 0/
{
	pending = 0;
}

profile:::profile-1000hz
{
	@out = lquantize(pending, 0, MAX_PENDING, 1);
}

profile:::tick-1sec
{
	/* print status */
	printf("%s %d", "LOAD", `hp_avenrun[0]);
}

??ALL 1
??DEVICE
??FILENAME
??MOUNT
